@@ -3,10 +3,8 @@
 <h3>Check config files</h3>
 <?php
 
-require_once 'include/rcube_html.inc';
-
-$read_main = is_readable('../config/main.inc.php');
-$read_db = is_readable('../config/db.inc.php');
+$read_main = is_readable(RCMAIL_CONFIG_DIR.'/main.inc.php');
+$read_db = is_readable(RCMAIL_CONFIG_DIR.'/db.inc.php');
 
 if ($read_main && !empty($RCI->config)) {
   $RCI->pass('main.inc.php');
@@ -29,22 +27,106 @@ else if (!$read_db) {
   $RCI->fail('db.inc.php', 'Unable to read file. Did you create the config files?');
 }
 
+if ($RCI->configured && ($messages = $RCI->check_config())) {
+  
+  if (is_array($messages['missing'])) {
+    echo '<h3 class="warning">Missing config options</h3>';
+    echo '<p class="hint">The following config options are not present in the current configuration.<br/>';
+    echo 'Please check the default config files and add the missing properties to your local config files.</p>';
+    
+    echo '<ul class="configwarings">';
+    foreach ($messages['missing'] as $msg) {
+      echo html::tag('li', null, html::span('propname', $msg['prop']) . ($msg['name'] ? ':&nbsp;' . $msg['name'] : ''));
+    }    
+    echo '</ul>';
+  }
+
+  if (is_array($messages['replaced'])) {
+    echo '<h3 class="warning">Replaced config options</h3>';
+    echo '<p class="hint">The following config options have been replaced or renamed. ';
+    echo 'Please update them accordingly in your config files.</p>';
+    
+    echo '<ul class="configwarings">';
+    foreach ($messages['replaced'] as $msg) {
+      echo html::tag('li', null, html::span('propname', $msg['prop']) .
+        ' was replaced by ' . html::span('propname', $msg['replacement']));
+    }
+    echo '</ul>';
+  }
+
+  if (is_array($messages['obsolete'])) {
+    echo '<h3>Obsolete config options</h3>';
+    echo '<p class="hint">You still have some obsolete or inexistent properties set. This isn\'t a problem but should be noticed.</p>';
+    
+    echo '<ul class="configwarings">';
+    foreach ($messages['obsolete'] as $msg) {
+      echo html::tag('li', null, html::span('propname', $msg['prop']) . ($msg['name'] ? ':&nbsp;' . $msg['name'] : ''));
+    }
+    echo '</ul>';
+  }
+  
+  echo '<p class="suggestion">OK, lazy people can download the updated config files here: ';
+  echo html::a(array('href' => './?_mergeconfig=main'), 'main.inc.php') . ' &nbsp;';
+  echo html::a(array('href' => './?_mergeconfig=db'), 'db.inc.php');
+  echo "</p>";
+  
+  
+  if (is_array($messages['dependencies'])) {
+    echo '<h3 class="warning">Dependency check failed</h3>';
+    echo '<p class="hint">Some of your configuration settings require other options to be configured or additional PHP modules to be installed</p>';
+    
+    echo '<ul class="configwarings">';
+    foreach ($messages['dependencies'] as $msg) {
+      echo html::tag('li', null, html::span('propname', $msg['prop']) . ': ' . $msg['explain']);
+    }
+    echo '</ul>';
+  }
+
+  
+}
+
+?>
+
+<h3>Check if directories are writable</h3>
+<p>RoundCube may need to write/save files into these directories</p>
+<?php
+
+if ($RCI->configured) {
+    $pass = false;
+
+    $dirs[] = $RCI->config['temp_dir'];
+    if($RCI->config['log_driver'] != 'syslog')
+      $dirs[] = $RCI->config['log_dir'];
+
+    foreach ($dirs as $dir) {
+        $dirpath = $dir{0} == '/' ? $dir : INSTALL_PATH . $dir;
+        if (is_writable(realpath($dirpath))) {
+            $RCI->pass($dir);
+            $pass = true;
+        }
+        else {
+            $RCI->fail($dir, 'not writeable for the webserver');
+        }
+        echo '<br />';
+    }
+    
+    if (!$pass)
+        echo '<p class="hint">Use <tt>chmod</tt> or <tt>chown</tt> to grant write privileges to the webserver</p>';
+}
+else {
+    $RCI->fail('Config', 'Could not read config files');
+}
+
 ?>
 
 <h3>Check configured database settings</h3>
 <?php
 
 $db_working = false;
-if (!empty($RCI->config)) {
-    if (!empty($RCI->config['db_backend']) && !empty($RCI->config['db_dsnw'])) {
+if ($RCI->configured) {
+    if (!empty($RCI->config['db_dsnw'])) {
 
-        echo 'Backend: ';
-        echo 'PEAR::' . strtoupper($RCI->config['db_backend']) . '<br />';
-
-        $_class = 'rcube_' . strtolower($RCI->config['db_backend']);
-        require_once 'include/' . $_class . '.inc';
-
-        $DB = new $_class($RCI->config['db_dsnw'], '', false);
+        $DB = new rcube_mdb2($RCI->config['db_dsnw'], '', false);
         $DB->db_connect('w');
         if (!($db_error_msg = $DB->is_error())) {
             $RCI->pass('DSN (write)');
@@ -53,10 +135,8 @@ if (!empty($RCI->config)) {
         }
         else {
             $RCI->fail('DSN (write)', $db_error_msg);
-            echo '<p class="hint">Make sure that the configured database extists and that the user as write privileges<br />';
+            echo '<p class="hint">Make sure that the configured database exists and that the user has write privileges<br />';
             echo 'DSN: ' . $RCI->config['db_dsnw'] . '</p>';
-            if ($RCI->config['db_backend'] == 'mdb2')
-              echo '<p class="hint">There are known problems with MDB2 running on PHP 4. Try setting <tt>db_backend</tt> to \'db\' instead</p>';
         }
     }
     else {
@@ -79,31 +159,40 @@ if ($db_working && $_POST['initdb']) {
 // test database
 if ($db_working) {
     $db_read = $DB->query("SELECT count(*) FROM {$RCI->config['db_table_users']}");
-    if (!$db_read) {
+    if ($DB->db_error) {
         $RCI->fail('DB Schema', "Database not initialized");
-        $db_working = false;
         echo '<p><input type="submit" name="initdb" value="Initialize database" /></p>';
+        $db_working = false;
     }
+  /*
+    else if (!$RCI->db_schema_check($update = !empty($_POST['updatedb']))) {
+        $RCI->fail('DB Schema', "Database schema differs");
+        
+        echo $update ? '<p class="warning">Failed to update the database schema! Please manually execute the SQL statements from the SQL/*.update.sql file on your database</p>' :
+          '<p><input type="submit" name="updatedb" value="Update schema now" /></p>';
+        $db_working = false;
+    }
+  */
     else {
         $RCI->pass('DB Schema');
+        echo '<br />';
     }
-    echo '<br />';
 }
 
 // more database tests
 if ($db_working) {
     // write test
-    $db_write = $DB->query("INSERT INTO {$RCI->config['db_table_cache']} (session_id, cache_key, data, user_id) VALUES (?, ?, ?, 0)", '1234567890abcdef', 'test', 'test');
-    $insert_id = $DB->insert_id($RCI->config['db_sequence_cache']);
-    
-    if ($db_write && $insert_id) {
+    $insert_id = md5(uniqid());
+    $db_write = $DB->query("INSERT INTO {$RCI->config['db_table_session']} (sess_id, created, ip, vars) VALUES (?, ".$DB->now().", '127.0.0.1', 'foo')", $insert_id);
+
+    if ($db_write) {
       $RCI->pass('DB Write');
-      $DB->query("DELETE FROM {$RCI->config['db_table_cache']} WHERE cache_id=?", $insert_id);
+      $DB->query("DELETE FROM {$RCI->config['db_table_session']} WHERE sess_id=?", $insert_id);
     }
     else {
       $RCI->fail('DB Write', $RCI->get_error());
     }
-    echo '<br />';    
+    echo '<br />';
     
     // check timezone settings
     $tz_db = 'SELECT ' . $DB->unixtimestamp($DB->now()) . ' AS tz_db';
@@ -137,11 +226,11 @@ if ($RCI->getprop('smtp_server')) {
   $pass = $RCI->getprop('smtp_pass', '(none)');
   
   if ($user == '%u') {
-    $user_field = new textfield(array('name' => '_user'));
-    $user = $user_field->show($_POST['_user']);
+    $user_field = new html_inputfield(array('name' => '_smtp_user'));
+    $user = $user_field->show($_POST['_smtp_user']);
   }
   if ($pass == '%p') {
-    $pass_field = new passwordfield(array('name' => '_pass'));
+    $pass_field = new html_passwordfield(array('name' => '_smtp_pass'));
     $pass = $pass_field->show();
   }
   
@@ -149,8 +238,8 @@ if ($RCI->getprop('smtp_server')) {
   echo "Password: $pass<br />";
 }
 
-$from_field = new textfield(array('name' => '_from', 'id' => 'sendmailfrom'));
-$to_field = new textfield(array('name' => '_to', 'id' => 'sendmailto'));
+$from_field = new html_inputfield(array('name' => '_from', 'id' => 'sendmailfrom'));
+$to_field = new html_inputfield(array('name' => '_to', 'id' => 'sendmailto'));
 
 ?>
 </p>
@@ -159,8 +248,7 @@ $to_field = new textfield(array('name' => '_to', 'id' => 'sendmailto'));
 
 if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to'])) {
   
-  require_once 'lib/rc_mail_mime.inc';
-  require_once 'include/rcube_smtp.inc';
+  require_once 'rcube_smtp.inc';
   
   echo '<p>Trying to send email...<br />';
   
@@ -168,8 +256,8 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
       preg_match('/^' . $RCI->email_pattern . '$/i', trim($_POST['_to']))) {
   
     $headers = array(
-      'From' => trim($_POST['_from']),
-      'To'  => trim($_POST['_to']),
+      'From'    => trim($_POST['_from']),
+      'To'      => trim($_POST['_to']),
       'Subject' => 'Test message from RoundCube',
     );
 
@@ -180,12 +268,14 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
     if ($RCI->getprop('smtp_server')) {
       $CONFIG = $RCI->config;
       
-      if (!empty($_POST['_user']))
-        $CONFIG['smtp_user'] = $_POST['_user'];
-      if (!empty($_POST['_pass']))
-        $CONFIG['smtp_pass'] = $_POST['_pass'];
-      
-      $mail_object  = new rc_mail_mime();
+      if (!empty($_POST['_smtp_user'])) {
+        $CONFIG['smtp_user'] = $_POST['_smtp_user'];
+      }
+      if (!empty($_POST['_smtp_pass'])) {
+        $CONFIG['smtp_pass'] = $_POST['_smtp_pass'];
+      }
+
+      $mail_object  = new rcube_mail_mime();
       $send_headers = $mail_object->headers($headers);
       
       $status = smtp_mail($headers['From'], $headers['To'],
@@ -214,10 +304,10 @@ if (isset($_POST['sendmail']) && !empty($_POST['_from']) && !empty($_POST['_to']
   else {
     $RCI->fail('SMTP send', 'Invalid sender or recipient');
   }
+  
+  echo '</p>';
 }
 
-echo '</p>';
-
 ?>
 
 <table>
@@ -242,15 +332,15 @@ echo '</p>';
 
 $default_hosts = $RCI->get_hostlist();
 if (!empty($default_hosts)) {
-  $host_field = new select(array('name' => '_host', 'id' => 'imaphost'));
+  $host_field = new html_select(array('name' => '_host', 'id' => 'imaphost'));
   $host_field->add($default_hosts);
 }
 else {
-  $host_field = new textfield(array('name' => '_host', 'id' => 'imaphost'));
+  $host_field = new html_inputfield(array('name' => '_host', 'id' => 'imaphost'));
 }
 
-$user_field = new textfield(array('name' => '_user', 'id' => 'imapuser'));
-$pass_field = new passwordfield(array('name' => '_pass', 'id' => 'imappass'));
+$user_field = new html_inputfield(array('name' => '_user', 'id' => 'imapuser'));
+$pass_field = new html_passwordfield(array('name' => '_pass', 'id' => 'imappass'));
 
 ?>
 
@@ -279,8 +369,6 @@ $pass_field = new passwordfield(array('name' => '_pass', 'id' => 'imappass'));
 
 if (isset($_POST['imaptest']) && !empty($_POST['_host']) && !empty($_POST['_user'])) {
   
-  require_once 'include/rcube_imap.inc';
-  
   echo '<p>Connecting to ' . Q($_POST['_host']) . '...<br />';
   
   $a_host = parse_url($_POST['_host']);
